''--------------------------------------------------------------------------------------------
''File......clock start 2.spin
''Purpose...start RTC in seperate cog and feed data back to serial terminal.
''              Also move clock setup to seperate method
''Author....Grant McFarland
''Email.....grantmcf@hotmail.com
''Started...04/10/2012
''Updated...4/20/2012
''Note:.....See Terms Below
''--------------------------------------------------------------------------------------------

CON
  _clkmode  = xtal1 + pll8x            'using low crystal gain, wind up 16x
  _xinfreq  = 10_000_000                 'external 5.0 MHz crystal on XI & X0

  Lcd_Pin   = 0                        'P0 LCD tx output pin
  up_Button1   = 8                     'Button 1 or Up button
  down_Button2 = 9                     'Button 2 or Down button
  next_Button3 = 10                    'Button 3 or Next button

Var
byte hour, minute, second, day, month, year, dow, ampm, hour12,ClockcogID

obj
  Clock : "RTC_new_cog"
  Lcd   : "Debug_Lcd"
  rtc   : "DS1302"
  DEBUG : "FullDuplexSerial"
Pub clockcog
Clock.Start (Lcd_Pin ,up_Button1, down_Button2, down_Button2)

     ' Initialize the LCD
  Lcd.init(Lcd_Pin, 19200, 4)                          
  Lcd.cls
  Lcd.home
  Lcd.cursor(0)
  waitcnt(clkfreq/1000 + cnt)                                     
  Lcd.backLight(true)
  waitcnt(clkfreq/1000 + cnt)

  'show marque
  Lcd.str(@Version)
  Lcd.gotoxy(0,1)
  Lcd.str(@Author)
  waitcnt(clkfreq * 5 + cnt) 

'This section displays data on serial terminal 
   
DEBUG.start(31, 30, 0, 38400)      'start pc serial communication
DEBUG.tx(16)                       'clear screen
repeat
  
  DEBUG.tx(1)                      'home position
  DEBUG.str(string("MM:DD:YY"))
  DEBUG.tx(11)                     'Clear to end of line
  DEBUG.tx(13)                     'Next line
  month:=(Clock.Getmonth)
  DEBUG.dec(month)
  DEBUG.str(string(":"))
  day:=(Clock.Getday)
  DEBUG.dec(day)
  DEBUG.str(string(":"))
  year:=(Clock.Getyear)
  DEBUG.dec(year)
  DEBUG.tx(11)                     'Clear to end of line
  DEBUG.tx(13)                     'Next line
  DEBUG.str(string("A/P:HH:MM:SS"))
  DEBUG.tx(11)                     'Clear to end of line
  DEBUG.tx(13)                     'Next line
  ampm:=(Clock.Getampm)
  if AMPM == 0
      DEBUG.str(string(" AM:"))
  else
      DEBUG.str(string(" PM:"))
  hour:=(Clock.Gethour)
  hour12:=(Clock.Gethour12)
  DEBUG.dec(hour12)
  DEBUG.str(string(":"))
  minute:= (Clock.Getminute)
  DEBUG.dec(minute)
  DEBUG.str(string(":"))
  second:=(Clock.Getsecond)
  DEBUG.dec(second)
  DEBUG.tx(11)                     'Clear to end of line
  DEBUG.tx(13)                     'Next line
  DEBUG.str(string("ClockcogID"))
  DEBUG.tx(11)                     'Clear to end of line
  DEBUG.tx(13)                     'Next line
  ClockcogID:=(Clock.GetClockcogID)
  DEBUG.dec(ClockcogID)
  dow:=(Clock.Getdow)

 'This section displays data on serial LCD
    Lcd.cls  'need to clear marque

    Lcd.gotoxy(0,0)
    lcd.str(lookup(dow:string("Sun"),string("Mon"),string("Tue"),string("Wed"){
    },string("Thu"),string("Fri"),string("Sat")))
    Lcd.str(string(": "))
    Lcd.decx(month,2)
    Lcd.str(string("-"))
    Lcd.decx(day,2)
    Lcd.str(string("-"))
    Lcd.decx(year,2)
    Lcd.gotoxy(0,1)
    if AMPM == 0
      Lcd.str(@AM)
    else
      Lcd.str(@PM)
    'Lcd.str(string("Time: ")) 
    Lcd.decx(hour12,2)
    Lcd.str(string(":"))
    Lcd.decx(minute,2)
    Lcd.str(string(":"))
    Lcd.decx(second,2)   
    'When button pressed button pin pulled low. 
    if not ina[up_Button1]             'Hold up button down for 2 sec to enter clock setup. 
      waitcnt(clkfreq/2 + cnt)         'Use up down and next buttons to set time and date
      SetDateTimeButton
     
    waitcnt(clkfreq + cnt)

PUB SetDateTimeButton  | colIndex
'routine to set ds1302 day time using 3 buttons, based on single button routine
'as suggested by Beau: http://forums.parallax.com/forums/default.aspx?f=25&m=468251

  lcd.cls
  Lcd.gotoxy(0,0)
  Lcd.str(@DateMask)                      'display date mask
  colIndex  := 0
  month     := ScanButton(month,1,12,colIndex)
  colIndex  := 3
  day       := ScanButton(day,1,31,colIndex)
  colIndex  := 6
  year      := ScanButton(year,10,30,colIndex)
  colIndex  := 9    
  dow       := ScanButton(dow,1,7,colIndex)
  lcd.cls
  Lcd.gotoxy(0,0)
  Lcd.str(@TimeMask)                        'display Time mask
  colIndex  := 1
  AMPM      := ScanButton(AMPM,0,1,colIndex)
  colIndex  := 4    
  hour      := ScanButton(hour,0,23,colIndex)  'hour is set in 24hr time and will auto config ampm.
  colIndex  := 7  
  minute    := ScanButton(minute,0,59,colIndex)
  second    := 0

  'update RTC with selected settings
  rtc.setDatetime(month, day, year, dow, hour, minute, second)
                     

  'clear SetDateTime menu/settings display
  Lcd.cls  
    
PUB ScanButton(varSet,loLim,hiLim,ci)|tmp 
'3-button routine to set date and time
'press and release up or down button to increment digit
'press and release next button to increment field

  dira [up_Button1]~            'sets pin to an input line with the ~ notation
  dira [down_Button2]~          'sets pin to an input line with the ~ notation
  dira [next_Button3]~          'sets pin to an input line with the ~ notation

   
  repeat
    lcd.gotoxy(ci,1)
    if ci <> 9            
      lcd.decx(varSet,2)
    else
      lcd.dec(varSet)                   'need single digit for dow
    if ina[up_Button1]==0               'check pin for high or low
      tmp := cnt
      waitcnt(clkfreq/2 + cnt)         'short delay to help debounce
      varSet++                          'increment value
      if varSet > hiLim                                 
        varSet := loLim
    if ina[down_Button2]==0             'check pin for high or low
      tmp := cnt
      waitcnt(clkfreq/2 + cnt)         'short delay to help debounce     
      varSet--                          'decrement value
      if varSet < loLim                                 
        varSet := hiLim
    if ina[next_Button3]==0             'check pin for high or low
       tmp := cnt
       waitcnt(clkfreq/2 + cnt)         'short delay to help debounce
       return varSet                    'return value
       waitcnt(clkfreq/5 + cnt)         'check button state 5 times/sec

DAT
  Version   byte "3 Button Clock v1", 0
  Author    byte "Author: GrantMcF", 0
  DateMask  byte "MM:DD:YY:d",0
  TimeMask  byte "A/P:HH:MM",0
  AM        byte "AM:",0
  PM        byte "PM:",0   
                      